#!/usr/bin/perl

use strict;

package File2DiVX3Pass;

use File2DiVX3Pass::interface;
use File2DiVX3Pass::signals;

my $true  = 1;
my $false = 0;

my %specs = ();
my $error = undef;
my $status = "stopped";
my $batchstatus = "stopped";
my $mencoderpid = undef;

## DEBUG
########
sub Debug
{
	my ($msg) = @_;
	print STDOUT "$msg\n";
}

## ERRORS
#########
sub SetError
{
	my ($msg) = @_;
	$error = $msg;
}
sub GetError
{
	return $error;
}

## CLEANSTRING
##############
sub CleanString
{
	my ($txt) = @_;
	$_ = $txt; s/\n//g; $txt = $_; return $txt;
}

## FILESPECS
############
sub ReadFileSpecs
{
	my ($file) = @_;
	%specs = ();

	my $cmd = "mplayer -identify -vc null -ac null -frames 0 \"$file\"";
	my $res = open( TMP, "$cmd 2>&1 |" );
	if( !defined($res) )
	{
		SetError( "Could not identify $file" );
		return $false;
	}
	while( <TMP> )
	{
		if( $_ =~ /ID_VIDEO_FPS/ )
		{
			my $str = CleanString( $_ );
			$str = substr( $str, 13 );
			$specs{'fps'} = $str;
		}
		elsif( $_ =~ /ID_LENGTH/ )
		{
			my $str = CleanString( $_ );
			$str = substr( $str, 10 );
			## nasty this one
			## realmedia has length = 0 -> will crash the app
			## while calculating the video bitrate
			## set it to 10 
			if( $str eq "0" ) { $str = "10"; }
			$specs{'seconds'} = $str;

		}
	}
	close( TMP );

	if( !defined($specs{'fps'}) )
	{
		SetError( "No frames/s found; make sure this is a valid media-file." );
		return $false;
	}
	elsif( !defined($specs{'seconds'}) )
	{
		SetError( "No length found; make sure this is a valid media-file." );
		return $false;
	}
	else { return $true; }
}
sub GetSpecFPS
{
	return $specs{'fps'};
}
sub GetSpecSeconds
{
	return $specs{'seconds'};
}


## CALCULATE VIDEOBITRATE
#########################
sub CalcVBrate
{
	my ($cdsize,$minutes,$audiobrate) = @_;
	my $videobrate = (((($cdsize*1024*8)/1000)*1024)-($audiobrate*$minutes*60)) / ($minutes*60) ;
	$videobrate = substr( $videobrate, 0, index( $videobrate, "." ) );
	# taken from divxcalc 0.5 (http://axljab.homelinux.org:8080/?m=sw&p=divxcalc)
}


## CALCULATE BPP
################
sub CalcBPP
{
	my ($videobrate,$width,$height,$fps) = @_;
	my $bpp = ($videobrate*1000) / ($width*$height*$fps);
	return $bpp;
}

## CHECK ENTRY
##############
sub CheckNumeralEntry
{
	my ($entry) = @_;
	my $txt = $entry->get_text;
	if( $txt eq "" )
	{ SetError("No value"); return $false; }
	elsif( $txt =~ /[A-Z,a-z]/ || $txt =~ /,;:=?\/+~%'`\[\]$*&\"\'(!)-_|@#{}<>\\/ )
	{ SetError("Only numerals"); return $false; }
	else{ return $true; }
}
sub CheckNonNumeralEntry
{
	my ($entry) = @_;
	my $txt = $entry->get_text;
	if( $txt eq "" )
	{ SetError("No value"); return $false; }
	else{ return $true; }
}

## ENCODE VIDEO
###############
sub EncodeVideo
{
	my ( 	$infile,
		$outfile,
		$audiobrate,
		$audiorate,
		$width,
		$height,
		$videobrate,
		$videocodec,
		$pass,
		$ofps,
		$widgets )   = @_;

	## make audio options
	my $cmdaudio = " -oac mp3lame -lameopts ";
	if( $audiorate eq "constant" ) { $cmdaudio .= "cbr:"; }
	elsif( $audiorate eq "average" ) { $cmdaudio .= "abr:"; }
	elsif( $audiorate eq "variable" ) { $cmdaudio .= "vbr:"; }
	$cmdaudio .= "br=$audiobrate ";

	## make vop options
	my $cmdvop = " -vop pp=de,scale=".$width.":".$height." ";

	## make codec options
	my $cmdcodec = "";
	if( $videocodec eq "divx4" )
	{
		$cmdcodec = " -ovc divx4 -divx4opts q=5:br=$videobrate:pass";
	}
	elsif( $videocodec eq "lavc" )
	{
		$cmdcodec = " -ovc lavc -lavcopts vcodec=mpeg4:vhq:vbitrate=$videobrate:vpass";
	}

	## make output FPS options
	my $cmdofps = "";
	if( $ofps != -1 )
	{
		$cmdofps = " -ofps $ofps ";
	}


	my @cmds = "";
	if( $pass eq "1-pass" )
	{
		$cmds[0] = "mencoder $cmdofps $cmdvop $cmdaudio $cmdcodec=1 -o \"$outfile\" \"$infile\"";
	}
	elsif( $pass eq "2-pass" )
	{
		$cmds[0] = "mencoder $cmdofps $cmdvop $cmdaudio $cmdcodec=1 -o /dev/null \"$infile\"";
		$cmds[1] = "mencoder $cmdofps $cmdvop $cmdaudio $cmdcodec=2 -o \"$outfile\" \"$infile\"";
	}
	elsif( $pass eq "3-pass" )
	{
		$cmds[0] = "mencoder $cmdofps -ovc frameno $cmdaudio -o frameno.avi \"$infile\"";
		$cmds[1] = "mencoder $cmdofps $cmdvop -oac copy $cmdcodec=1 -o /dev/null \"$infile\"";
		$cmds[2] = "mencoder $cmdofps $cmdvop -oac copy $cmdcodec=2 -o \"$outfile\" \"$infile\"";
	}

	Clean();
	for( my $i=0; $i<@cmds; ++$i )
	{
		if( GetStatus() eq "started" )
		{
			if( !EncodePass( $cmds[$i], $i+1, $widgets ) )
			{
				SetError( GetError()." while encoding pass ".($i+1) );
				return $false;
			}
		}
		else
		{
			SetError( "Encoding stopped" );
			return $false;
		}
	}
	Clean();

	return $true;
}

sub EncodePass
{
	my ($cmd,$pass,$widgets) = @_;

	$widgets->{'lblActions'}->set_text( "Encoding pass $pass" );
	$widgets->{pbarBar}->set_value( 0 );
	while (Gtk->events_pending()) {Gtk->main_iteration()};

	my $specfps = GetSpecFPS();
	my $specsec = GetSpecSeconds();

	my $res = open( TMP, "$cmd 2>&1 |" );
	if( !defined($res) )
	{
		SetError( "Could not encode pass $pass" );
		return $false;
	}
	SetMencoderPID( $res );

	$/="\r"; ## from http://acidrip.thirtythreeandathird.net/
	while( <TMP> )
	{
		#Pos:  49.0s   1179f ( 1%) 232fps Trem:   8min  69mb  A-V:0.084 [0:115]
		if( $_ =~ /Skipping/ ) {  }
		elsif( $_ =~ /duplicate/ ) {  }
		elsif( $_ =~ /Pos:/ && $_ =~ /Trem:/ )
		{
			my $numframes = substr( $_, 11, 7 );
			my $pct = substr( $_, 21, 2 );
			my $fps = substr( $_, 25, 4 ); $fps =~ s/ //g;
			my $todotime = GetHour( $specsec - ($numframes/$specfps) );
			my $donetime = GetHour( $numframes / $specfps );

			my $str = sprintf( "Encoding pass %d - fps: %4d - done: %s - todo: %s", $pass, $fps, $donetime, $todotime );

			if ( $widgets->{'pbarBar'}->get_value ne $pct )
			{
				$widgets->{'pbarBar'}->set_value( $pct ) ;
			}
		#	if( $widgets->{'lblActions'}->get ne "Encoding pass $pass\nfps  : $fps\ndone : $donetime\ntodo : $todotime" )
		#	{
		#		$widgets->{'lblActions'}->set_text( "Encoding pass $pass\nfps  : $fps\ndone : $donetime\ntodo : $todotime" )
		#	}
			if( $widgets->{'lblActions'}->get ne $str )
			{
				$widgets->{'lblActions'}->set_text( $str )
			}

			while (Gtk->events_pending()) { Gtk->main_iteration()};
		}
		while (Gtk->events_pending()) { Gtk->main_iteration()};
	}
	$/="\n"; ## from http://acidrip.thirtythreeandathird.net/
	close( TMP );
	$widgets->{pbarBar}->set_value( 100 );
	SetMencoderPID( undef );
	return $true;
}

sub Clean
{
	unlink "frameno.avi", "divx2pass.log", "analyse.log", "c:\\trace_b.txt";
}

sub GetHour
{
	my ($s) = @_;

	my $neg = $false;
	if( substr( $s, 0, 1 ) eq "-" ) { $neg = $true; $s = substr( $s, 1 ); }

	my $h = 0;
	my $m = 0;
	if( $s / 3600 > 0 )
	{ $h = $s / 3600; $h = substr($h,0,index($h,".")); $s -= $h * 3600; }
	if( $s / 60 > 0 )
	{ $m = $s / 60; $m = substr($m,0,index($m,".")); $s -= $m * 60; }

	$s = substr($s,0,index($s,"."));

	if( length($h) == 1 ) { $h = "0".$h; }
	if( length($m) == 1 ) { $m = "0".$m; }
	if( length($s) == 1 ) { $s = "0".$s; }
	
	if( $neg ) { return "-$h:$m:$s"; }
	else { return "$h:$m:$s"; }
}


## STATUS
sub SetStatus
{
	my ($stat) = @_;
	$status = $stat;
}
sub GetStatus
{
	return $status;
}

## BATCHSTATUS
sub SetBatchStatus
{
	my ($stat) = @_;
	$batchstatus = $stat;
}
sub GetBatchStatus
{
	return $batchstatus;
}

## MENCODER PID
sub SetMencoderPID
{
	my ($pid) = @_;
	$mencoderpid = $pid;
}
sub GetMencoderPID
{
	return $mencoderpid;
}

1;
