#!/usr/bin/perl

package File2DiVX3Pass;
use strict;

use File2DiVX3Pass::helpers;

my $true  = 1;
my $false = 0;


## close the main window
sub wdwMainClose
{
	my ($wdw) = @_;
	$wdw->destroy();
	Gtk->exit( $false );
}

## click select file for input
sub btnInputClicked
{
	my ($button,$widgets) = @_;

	## create a file selection widgets + add signals
	$widgets->{'fsInput'} = new Gtk::FileSelection( "Select file to encode" );
	$widgets->{'fsInput'}->signal_connect( "delete_event", \&fsInputClose );
	$widgets->{'fsInput'}->ok_button->signal_connect( "clicked", \&fsInputOkButtonClicked, $widgets );
	$widgets->{'fsInput'}->cancel_button->signal_connect( "clicked", \&fsInputCancelButtonClicked, $widgets );
	$widgets->{'fsInput'}->show;
}
## close the file selection widgets for input files
sub fsInputClose
{
	my ($fsel) = @_;
	$fsel->destroy();
}
## cancel clicked for file selection for input files
sub fsInputCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsInput'}->destroy();
}
## ok clicked for file selection for input files
sub fsInputOkButtonClicked
{
	my ($button,$widgets) = @_;
	## get filename
	$widgets->{'fsInput'}->hide();
	my $txt = $widgets->{fsInput}->get_filename();
	$widgets->{'fsInput'}->destroy();

	SetInputFile( $txt,$widgets );
}

## read the specs from the input file, and set the widgets
sub SetInputFile
{
	my ($filename,$widgets) = @_;

	$widgets->{'lblActions'}->set_text( "Checking file..." );
	## get the specs of the selected files
	if( !ReadFileSpecs( $filename ) )
	{
		$widgets->{'lblActions'}->set_text( GetError() );
		return;
	}

	## check the size of the CD
	my $cdsize = 0;
	if( !CheckNumeralEntry($widgets->{'entrySize'}) )
	{
		$widgets->{'lblActions'}->set_text( GetError()." for CD size" );
		return;
	}
	else { $cdsize = $widgets->{'entrySize'}->get_text; }

	## check the audio bitrate
	my $audiobrate = 0;
	if( !CheckNumeralEntry( $widgets->{'cmbAudioBrate'}->entry ) )
	{
		$widgets->{'lblActions'}->set_text( GetError()." for Audio MP3 bitrate" );
		return;
	}
	else { $audiobrate = $widgets->{'cmbAudioBrate'}->entry->get_text; }

	## put the filename in widget entryInput
	$widgets->{'entryInput'}->set_text( $filename );

	## put the filename in widget entryOutput
	my $f = substr( $filename, rindex($filename,"/")+1 );
	$f = substr( $f, 0, rindex($f,".") ).".avi";
	$widgets->{'entryOutput'}->set_text( $f );

	## put minutes in widget entryLength
	my $sec = GetSpecSeconds();
	my $min = $sec / 60;
	$min = substr( $min, 0, index($min,".")+2 );
	## put the length in widget entryLength
	$widgets->{'entryLength'}->set_text( $min );

	## calculate video bitrate
	my $fps = GetSpecFPS();
	my $videobrate = CalcVBrate( $cdsize, $min, $audiobrate );
	## put the video brate in widget entryVideoBrate
	$widgets->{'entryVideoBrate'}->set_text( $videobrate );
}

## size of CD is changed
sub entrySizeActivated
{
	my ($entry,$widgets) = @_;

	$widgets->{'lblActions'}->set_text( "" );
	if( $widgets->{'entryInput'}->get_text ne "" )
	{
		## check size of the CD
		my $cdsize = 0;
		if( !CheckNumeralEntry($widgets->{'entrySize'}) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for CD size" );
			return;
		}
		else { $cdsize = $widgets->{'entrySize'}->get_text; }
		## check audio bitrate
		my $audiobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'cmbAudioBrate'}->entry ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Audio MP3 bitrate" );
			return;
		}
		else { $audiobrate = $widgets->{'cmbAudioBrate'}->entry->get_text; }
		## check length of the file (in minutes)
		my $min = 0;
		if( !CheckNumeralEntry( $widgets->{'entryLength'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for File length" );
			return;
		}
		else { $min = $widgets->{'entryLength'}->get_text; }

		## calculate video bitrate
		my $videobrate = CalcVBrate( $cdsize, $min, $audiobrate );
		## put the video brate in widget entryVideoBrate
		$widgets->{'entryVideoBrate'}->set_text( $videobrate );
	}
}

## audio bitrate has changed
sub cmbAudioBrateEntryChanged
{
	my ($entry,$widgets) = @_;

	$widgets->{'lblActions'}->set_text( "" );
	if( $widgets->{'entryInput'}->get_text ne "" )
	{
		## check size of the CD
		my $cdsize = 0;
		if( !CheckNumeralEntry($widgets->{'entrySize'}) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for CD size" );
			return;
		}
		else { $cdsize = $widgets->{'entrySize'}->get_text; }
		## check audio bitrate
		my $audiobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'cmbAudioBrate'}->entry ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Audio MP3 bitrate" );
			return;
		}
		else { $audiobrate = $widgets->{'cmbAudioBrate'}->entry->get_text; }
		## check length of file (in minutes)
		my $min = 0;
		if( !CheckNumeralEntry( $widgets->{'entryLength'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for File length" );
			return;
		}
		else { $min = $widgets->{'entryLength'}->get_text; }

		## calculate video bitrate
		my $videobrate = CalcVBrate( $cdsize, $min, $audiobrate );
		## put the video brate in widget entryVideoBrate
		$widgets->{'entryVideoBrate'}->set_text( $videobrate );
	}
}

## video bitrate has changed
sub entryVideoBrateChanged
{
	my ($entry,$widgets) = @_;

	if( $widgets->{'entryInput'}->get_text ne "" )
	{
		## check scale width
		my $fps = 0;
		if( $widgets->{'chkbtnOutputFPS'}->active )
		{ $fps = $widgets->{'entryOutputFPS'}->get_text; }
		else { $fps = GetSpecFPS(); }

		my $width = 0;
		if( !CheckNumeralEntry( $widgets->{'entryScaleWidth'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Scale width" );
			return;
		}
		else { $width = $widgets->{'entryScaleWidth'}->get_text; }
		## check scale height
		my $height = 0;
		if( !CheckNumeralEntry( $widgets->{'entryScaleHeight'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Scale Height" );
			return;
		}
		else { $height = $widgets->{'entryScaleHeight'}->get_text; }
		## check video bitrate
		my $videobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'entryVideoBrate'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Video bitrate" );
			return;
		}
		else { $videobrate = $widgets->{'entryVideoBrate'}->get_text; }

		$widgets->{'lblActions'}->set_text( "" );
		## calculate bpp
		my $bpp = CalcBPP( $videobrate, $width, $height, $fps );
		$bpp = substr( $bpp, 0, index($bpp,".")+4);
		## put bpp in widget entryVideoBPP
		$widgets->{'entryVideoBPP'}->set_text( $bpp );

	}
}

## scale width has changed
sub entryScaleWidthActivated
{
	my ($entry, $widgets) = @_;

	my $sch = 0;
	my $scw = 0;

	## calc weither it's 4:3 or 16:9
	$widgets->{'lblActions'}->set_text( "" );
	if( $widgets->{'rbtnScale43'}->active )
	{
  		$sch = 4;
		$scw = 3;
	}
	elsif( $widgets->{'rbtnScale169'}->active )
	{
 		$sch = 16;
		$scw = 9;
	}

	## check scale width
	my $w = 0;
	if( !CheckNumeralEntry( $widgets->{'entryScaleWidth'} ) )
	{
		$widgets->{'lblActions'}->set_text( GetError()." for Scale width" );
		return;
	}
	else { $w = $widgets->{'entryScaleWidth'}->get_text; }

	my $h = ($w/$sch)*$scw;
	entrySizeActivated( undef, $widgets );
	$widgets->{'entryScaleHeight'}->set_text( $h );
	entryVideoBrateChanged( undef, $widgets );
}

## scale height has changed
sub entryScaleHeightActivated
{
	my ($entry, $widgets) = @_;

	my $sch = 0;
	my $scw = 0;

	## calc weither it's 4:3 or 16:9
	$widgets->{'lblActions'}->set_text( "" );
	if( $widgets->{'rbtnScale43'}->active )
	{
  		$sch = 4;
		$scw = 3;
	}
	elsif( $widgets->{'rbtnScale169'}->active )
	{
 		$sch = 16;
		$scw = 9;
	}

	## check scale height
	my $h = 0;
	if( !CheckNumeralEntry( $widgets->{'entryScaleHeight'} ) )
	{
		$widgets->{'lblActions'}->set_text( GetError()." for Scale height" );
		return;
	}
	else { $h = $widgets->{'entryScaleHeight'}->get_text; }

	my $w = ($h/$scw)*$sch;
	entrySizeActivated( undef, $widgets );
	$widgets->{'entryScaleWidth'}->set_text( $w );
	entryVideoBrateChanged( undef, $widgets );
}

## 4:3 has toggled
sub rbtnScale43Toggled
{
	my ($rbtn,$widgets) = @_;
	$widgets->{'lblActions'}->set_text( "" );
	entryScaleWidthActivated( undef, $widgets );
}

## button start is clicked
sub btnStartClicked
{
	my ($btn, $widgets) = @_;

	$widgets->{'lblActions'}->set_text( "" );
	if( CheckNonNumeralEntry ( $widgets->{'entryInput'} ) )
	{
		## check output
		my $outfile = "out.divx.avi";
		if( !CheckNonNumeralEntry ( $widgets->{'entryOutput'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Output file" );
		}
		else { $outfile = $widgets->{'entryOutput'}->get_text; }

		## check audio bitrate
		my $audiobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'cmbAudioBrate'}->entry ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Audio MP3 bitrate" );
			return;
		}
		else { $audiobrate = $widgets->{'cmbAudioBrate'}->entry->get_text; }

		## check audio rate
		my $audiorate = 0;
		if( !CheckNonNumeralEntry( $widgets->{'cmbAudioRate'}->entry ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Bitrate" );
			return;
		}
		else { $audiorate = $widgets->{'cmbAudioRate'}->entry->get_text; }

		## check scale width
		my $width = 0;
		if( !CheckNumeralEntry( $widgets->{'entryScaleWidth'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Scale width" );
			return;
		}
		else { $width = $widgets->{'entryScaleWidth'}->get_text; }

		## check scale height
		my $height = 0;
		if( !CheckNumeralEntry( $widgets->{'entryScaleHeight'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Scale height" );
			return;
		}
		else { $height = $widgets->{'entryScaleHeight'}->get_text; }

		## check video bitrate
		my $videobrate = 0;
		if( !CheckNumeralEntry( $widgets->{'entryVideoBrate'} ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Video bitrate" );
			return;
		}
		else { $videobrate = $widgets->{'entryVideoBrate'}->get_text; }

		## check video codec
		my $videocodec = 0;
		if( !CheckNonNumeralEntry( $widgets->{'cmbCodec'}->entry ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Video codec" );
			return;
		}
		else { $videocodec = $widgets->{'cmbCodec'}->entry->get_text; }

		## check pass
		my $pass = 0;
		if( !CheckNonNumeralEntry( $widgets->{'cmbPass'}->entry ) )
		{
			$widgets->{'lblActions'}->set_text( GetError()." for Pass" );
			return;
		}
		else { $pass = $widgets->{'cmbPass'}->entry->get_text; }

		## check ofps
		my $ofps = -1;
		if( $widgets->{'chkbtnOutputFPS'}->active )
		{
			if( !CheckNonNumeralEntry( $widgets->{'entryOutputFPS'} ) )
			{
				$widgets->{'lblActions'}->set_text( GetError()." for Output FPS" );
				return;
			}
			else { $ofps = $widgets->{'entryOutputFPS'}->get_text; }
		}

		my $infile = $widgets->{'entryInput'}->get_text;
		## start the encoding
		if( GetStatus() ne "started" )
		{
			SetStatus( "started" );
			$widgets->{'btnStart'}->set_sensitive( $false );
			$widgets->{'btnStop'}->set_sensitive( $true );
			while (Gtk->events_pending()) { Gtk->main_iteration() };
			if( EncodeVideo( $infile, $outfile, $audiobrate, $audiorate, $width, $height, $videobrate, $videocodec, $pass, $ofps, $widgets ) )
			{
				$widgets->{'btnStart'}->set_sensitive( $true );
				$widgets->{'btnStop'}->set_sensitive( $false );
				$widgets->{'lblActions'}->set_text( "Done" );
			}
			else
			{
				$widgets->{'lblActions'}->set_text( GetError() );
			}
			SetStatus( "stopped" );
		}
		else { $widgets->{'lblActions'}->set_text( "Already busy !!" ) };
	}
}

## button stop is clicked
sub btnStopClicked
{
	my ($button,$widgets) = @_;

	if( GetStatus() eq "started" )
	{
		my $pid = GetMencoderPID();
		if( defined( $pid ) )
		{
			## kill the menoder PID
			## release buttons
			++$pid;
			kill 9, $pid;
			$widgets->{'btnStart'}->set_sensitive( $true );
			$widgets->{'btnStop'}->set_sensitive( $false );
			SetStatus( "stopped" );
		}
	}
}

## click on menu about
sub MenuAbout
{
	my ($version) = @_;

	my $wdw = new Gtk::Window( "toplevel" );
	$wdw->set_title( "About" );
	$wdw->set_policy( $false, $true, $false );
	$wdw->set_position( "center" );
	$wdw->border_width( 5 );
	$wdw->set_modal( $true );

	my $frm = new Gtk::Frame();
	$wdw->add( $frm );

	my $lbl = new Gtk::Label( "  File2DiVX3Pass $version  \n  by Rien (rien\@evilnerds.be)  \n  Because an idle CPU is a terrible thing to waste  " );
	$frm->add( $lbl );

	$wdw->show_all;

}

## click on menu new
sub MenuNew
{
	my ($widgets) = @_;

	if( GetStatus() ne "started" )
	{
		## clear input and output
		$widgets->{'entryInput'}->set_text( "" );
		$widgets->{'entryOutput'}->set_text( "out.divx.avi" );

		## reset audio bitrate and rate
		$widgets->{'cmbAudioBrate'}->list->select_item( 8 );
		$widgets->{'cmbAudioRate'}->list->select_item( 1 );

		## clear file length
		$widgets->{'entryLength'}->set_text( "" );
		## reset cd size
		$widgets->{'entrySize'}->set_text( "700" );
		## reset scaling
		$widgets->{'entryScaleWidth'}->set_text( "480" );
		$widgets->{'entryScaleHeight'}->set_text( "360" );
		## reset aspect ratio
		$widgets->{'rbtnScale43'}->active;
		## clear video bitrate
		$widgets->{'entryVideoBrate'}->set_text( "" );
		$widgets->{'entryVideoBPP'}->set_text( "" );

		## reset codec
		$widgets->{'cmbCodec'}->list->select_item( 0 );
		## reset pass
		$widgets->{'cmbPass'}->list->select_item( 2 );

		## reset output FPS
		$widgets->{'chkbtnOutputFPS'}->set_active( $false );
		$widgets->{'entryOutputFPS'}->set_sensitive( $false );
		$widgets->{'entryOutputFPS'}->set_text( "25" );

	}
}

## click on menu save
sub MenuSave
{
	my ($widgets) = @_;

	if( GetStatus() ne "started" )
	{
		## start with a file selection where to save to
		$widgets->{'fsSave'} = new Gtk::FileSelection( "Save project" );
		$widgets->{'fsSave'}->signal_connect( "delete_event", \&fsSaveClose );
		$widgets->{'fsSave'}->ok_button->signal_connect( "clicked", \&fsSaveOkButtonClicked, $widgets );
		$widgets->{'fsSave'}->cancel_button->signal_connect( "clicked", \&fsSaveCancelButtonClicked, $widgets );
		$widgets->{'fsSave'}->show;
	}
}
## close the file selection for saving the project
sub fsSaveClose
{
	my ($fsel) = @_;
	$fsel->destroy;
}
## cancel saving the project
sub fsSaveCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsSave'}->destroy;
}
## save the project
sub fsSaveOkButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsSave'}->hide;
	my $txt = $widgets->{'fsSave'}->get_filename;
	$widgets->{'fsSave'}->destroy;

	my $res = open(FP,">$txt");
	if( !defined($res) )
	{
		$widgets->{'lblActions'}->set_text( "Could not save project" );
		return;
	}

	print FP "entryVideoBrate=".$widgets->{'entryVideoBrate'}->get_text()."\n";
	print FP "entryVideoBPP=".$widgets->{'entryVideoBPP'}->get_text()."\n";
	print FP "entryInput=".$widgets->{'entryInput'}->get_text()."\n";
	print FP "entryOutput=".$widgets->{'entryOutput'}->get_text()."\n";
	print FP "cmbAudioBrate=".$widgets->{'cmbAudioBrate'}->entry->get_text()."\n";
	print FP "cmbAudioRate=".$widgets->{'cmbAudioRate'}->entry->get_text()."\n";
	print FP "entryLength=".$widgets->{'entryLength'}->get_text()."\n";
	print FP "entrySize=".$widgets->{'entrySize'}->get_text()."\n";
	print FP "entryScaleWidth=".$widgets->{'entryScaleWidth'}->get_text()."\n";
	print FP "entryScaleHeight=".$widgets->{'entryScaleHeight'}->get_text()."\n";
	if( $widgets->{'rbtnScale43'}->active )
	{ print FP "rbtnScale43=true\n"; }
	else { print FP "rbtnScale43=false\n"; }
	print FP "cmbCodec=".$widgets->{'cmbCodec'}->entry->get_text()."\n";
	print FP "cmbPass=".$widgets->{'cmbPass'}->entry->get_text()."\n";
	if( $widgets->{'chkbtnOutputFPS'}->active )
	{ print FP "chkbtnOutputFPS=true\n"; }
	print FP "entryOutputFPS=".$widgets->{'entryOutputFPS'}->get_text()."\n";
	close( FP );

	$widgets->{'lblActions'}->set_text( "Project saved" );
}

## click menu open
sub MenuOpen
{
	my ($widgets) = @_;

	if( GetStatus() ne "started" )
	{
		## file selection for opening the project
		$widgets->{'fsOpen'} = new Gtk::FileSelection( "Open project" );
		$widgets->{'fsOpen'}->signal_connect( "delete_event", \&fsOpenClose );
		$widgets->{'fsOpen'}->ok_button->signal_connect( "clicked", \&fsOpenOkButtonClicked, $widgets );
		$widgets->{'fsOpen'}->cancel_button->signal_connect( "clicked", \&fsOpenCancelButtonClicked, $widgets );
		$widgets->{'fsOpen'}->show;
	}
}
## close the file selection for opening project
sub fsOpenClose
{
	my ($fsel) = @_;
	$fsel->destroy;
}
## cancel opening the project
sub fsOpenCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsOpen'}->destroy;
}
## open project
sub fsOpenOkButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsOpen'}->hide;
	my $txt = $widgets->{'fsOpen'}->get_filename;
	$widgets->{'fsOpen'}->destroy;

	ReadProject( $txt, $widgets );

	$widgets->{'lblActions'}->set_text( "Project opened" );
}
## read project file and put in widgets
sub ReadProject
{
	my ($filename,$widgets) = @_;
	my $res = open(FP,"<$filename");
	if( !defined($res) )
	{
		$widgets->{'lblActions'}->set_text( "Could not open project $filename" );
		return;
	}
	while( <FP> )
	{
		my ($k,$v) = split( "=", $_ );
		$v = CleanString( $v );

		if( $k eq "entryVideoBrate" )
		{ $widgets->{'entryVideoBrate'}->set_text( $v ); }
		if( $k eq "entryVideoBPP" )
		{ $widgets->{'entryVideoBPP'}->set_text( $v ); }
		if( $k eq "entryInput" )
		{ $widgets->{'entryInput'}->set_text( $v ); ReadFileSpecs( $v ); }
		if( $k eq "entryOutput" )
		{ $widgets->{'entryOutput'}->set_text( $v ); }
		if( $k eq "cmbAudioBrate" )
		{ $widgets->{'cmbAudioBrate'}->entry->set_text( $v ); }
		if( $k eq "cmbAudioRate" )
		{ $widgets->{'cmbAudioRate'}->entry->set_text( $v ); }
		if( $k eq "entryLength" )
		{ $widgets->{'entryLength'}->set_text( $v ); }
		if( $k eq "entrySize" )
		{ $widgets->{'entrySize'}->set_text( $v ); }
		if( $k eq "entryScaleWidth" )
		{ $widgets->{'entryScaleWidth'}->set_text( $v ); }
		if( $k eq "entryScaleHeight" )
		{ $widgets->{'entryScaleHeight'}->set_text( $v ); }
		if( $k eq "rbtnScale43" && $v eq "true" )
		{ $widgets->{'rbtnScale43'}->active; }
		if( $k eq "cmbCodec" )
		{ $widgets->{'cmbCodec'}->entry->set_text( $v ); }
		if( $k eq "cmbPass" )
		{ $widgets->{'cmbPass'}->entry->set_text( $v ); }
		if( $k eq "chkbtnOutputFPS" && $v eq "true" )
		{ $widgets->{'chkbtnOutputFPS'}->set_active( $true ); $widgets->{'entryOutputFPS'}->set_sensitive( $true ); }
		if( $k eq "entryOutputFPS" )
		{ $widgets->{'entryOutputFPS'}->set_text( $v ); }
	}
	close( FP );
}



## click menu batch encode
sub MenuBatchEncode
{
	my ($widgets) = @_;

	if( GetStatus() ne "started" )
	{
		## show window for batch encoding
		## if this widgets does NOT exist
		## else show it
		if( !defined($widgets->{'wdwBatchMain'}) )
		{
			$widgets->{'wdwBatchMain'} = new Gtk::Window( "toplevel" );
			$widgets->{'wdwBatchMain'}->set_title( "Batch encode" );
			$widgets->{'wdwBatchMain'}->set_policy( $false, $true, $false );
			$widgets->{'wdwBatchMain'}->set_position( "center" );
			$widgets->{'wdwBatchMain'}->border_width( 5 );

			$widgets->{'vbBatchMain'} = new Gtk::VBox( $false, 2 );
			$widgets->{'wdwBatchMain'}->add( $widgets->{'vbBatchMain'} );

			$widgets->{'frmBatchMain'} = new Gtk::Frame();
			$widgets->{'vbBatchMain'}->add( $widgets->{'frmBatchMain'} );
			$widgets->{'vbBatchMain'}->set_child_packing( $widgets->{'frmBatchMain'}, $false, $true, 2, "start" );

			$widgets->{'hbBatchMain'} = new Gtk::HBox( $false, 2 );
			$widgets->{'frmBatchMain'}->add( $widgets->{'hbBatchMain'} );

				$widgets->{'swBatchList'} = new Gtk::ScrolledWindow( undef, undef );
			$widgets->{'hbBatchMain'}->add( $widgets->{'swBatchList'} );
			$widgets->{'hbBatchMain'}->set_child_packing( $widgets->{'swBatchList'}, $false, $true, 2, "start" );
				#$widgets->{'swBatchList'}->border_width( 5 );
				$widgets->{'swBatchList'}->set_policy( "automatic","automatic" );
				$widgets->{'lstBatchList'} = new Gtk::List();
				$widgets->{'swBatchList'}->add_with_viewport( $widgets->{'lstBatchList'} );
				$widgets->{'lstBatchList'}->set_selection_mode( "single" );
				$widgets->{'swBatchList'}->set_usize( 200, 200 );

				$widgets->{'vbBatchButtons'} = new Gtk::VBox( $false, 2 );
			$widgets->{'hbBatchMain'}->add( $widgets->{'vbBatchButtons'} );
			$widgets->{'hbBatchMain'}->set_child_packing( $widgets->{'vbBatchButtons'}, $false, $true, 2, "start" );
					$widgets->{'btnBatchAddProject'} = new Gtk::Button( "Add project..." );
				$widgets->{'vbBatchButtons'}->add( $widgets->{'btnBatchAddProject'} );
				$widgets->{'vbBatchButtons'}->set_child_packing( $widgets->{'btnBatchAddProject'}, $false, $true, 2, "start" );
					$widgets->{'btnBatchRemoveProject'} = new Gtk::Button( "Remove project" );
				$widgets->{'vbBatchButtons'}->add( $widgets->{'btnBatchRemoveProject'} );
				$widgets->{'vbBatchButtons'}->set_child_packing( $widgets->{'btnBatchRemoveProject'}, $false, $true, 2, "start" );
					$widgets->{'btnBatchRemoveProjects'} = new Gtk::Button( " Remove projects " );
				$widgets->{'vbBatchButtons'}->add( $widgets->{'btnBatchRemoveProjects'} );
				$widgets->{'vbBatchButtons'}->set_child_packing( $widgets->{'btnBatchRemoveProjects'}, $false, $true, 2, "start" );
					$widgets->{'btnBatchStop'} = new Gtk::Button( "STOP" );
				$widgets->{'vbBatchButtons'}->add( $widgets->{'btnBatchStop'} );
				$widgets->{'vbBatchButtons'}->set_child_packing( $widgets->{'btnBatchStop'}, $false, $true, 2, "end" );
					$widgets->{'btnBatchStart'} = new Gtk::Button( "START" );
				$widgets->{'vbBatchButtons'}->add( $widgets->{'btnBatchStart'} );
				$widgets->{'vbBatchButtons'}->set_child_packing( $widgets->{'btnBatchStart'}, $false, $true, 2, "end" );


			$widgets->{'hbBatchActions'} = new Gtk::HBox( $false, 1 );
			$widgets->{'vbBatchMain'}->add( $widgets->{'hbBatchActions'} );
			$widgets->{'vbBatchMain'}->set_child_packing( $widgets->{'hbBatchActions'}, $false, $false, 2, "start" );
				$widgets->{'lblBatchActions'} = new Gtk::Label( "" );
				$widgets->{'lblBatchActions'}->set_justify( "left" );
				$widgets->{'hbBatchActions'}->add( $widgets->{'lblBatchActions'} );
				$widgets->{'hbBatchActions'}->set_child_packing( $widgets->{'lblBatchActions'}, $false, $false, 2, "start" );



			## signals
			$widgets->{'wdwBatchMain'}->signal_connect( "delete_event", \&wdwBatchMainClosed, $widgets );
			$widgets->{'btnBatchAddProject'}->signal_connect( "clicked", \&btnBatchAddProjectClicked, $widgets );
			$widgets->{'btnBatchRemoveProject'}->signal_connect( "clicked", \&btnBatchRemoveProjectClicked, $widgets );
			$widgets->{'btnBatchRemoveProjects'}->signal_connect( "clicked", \&btnBatchRemoveProjectsClicked, $widgets );
			$widgets->{'btnBatchStart'}->signal_connect( "clicked", \&btnBatchStartClicked, $widgets );
			$widgets->{'btnBatchStop'}->signal_connect( "clicked", \&btnBatchStopClicked, $widgets );

			$widgets->{'wdwBatchMain'}->show_all();
		}
		else
		{
			while (Gtk->events_pending()) {Gtk->main_iteration()};
			$widgets->{'wdwBatchMain'}->show_all;
			while (Gtk->events_pending()) {Gtk->main_iteration()};
		}
	}
	else
	{
		while (Gtk->events_pending()) {Gtk->main_iteration()};
		$widgets->{'wdwBatchMain'}->show_all();
		while (Gtk->events_pending()) {Gtk->main_iteration()};
	}
}

## close the batch encode window
## hide it is what we want ... if filled in, 2nd time use start batch encode
## the list will still be there
## bug here : no debug last line -> hiding the widgets will destroy it in some kind
sub wdwBatchMainClosed
{
	my ($wdw, $widgets) = @_;

	$widgets->{'wdwBatchMain'}->hide;
	Debug( "" );
}

## add project to list
sub btnBatchAddProjectClicked
{
	my ($button,$widgets) = @_;

	## start file selection
	## multiple selection !!
	$widgets->{'fsBatchAddProject'} = new Gtk::FileSelection( "Select project to add" );
	$widgets->{'fsBatchAddProject'}->signal_connect( "delete_event", \&fsBatchAddProjectClose );
	$widgets->{'fsBatchAddProject'}->ok_button->signal_connect( "clicked", \&fsBatchAddProjectOkButtonClicked, $widgets );
	$widgets->{'fsBatchAddProject'}->cancel_button->signal_connect( "clicked", \&fsBatchAddProjectCancelButtonClicked, $widgets );
	$widgets->{'fsBatchAddProject'}->file_list->set_selection_mode( "multiple" );
	$widgets->{'fsBatchAddProject'}->show;
}
## add project to list closed
sub fsBatchAddProjectClose
{
	my ($filesel) = @_;
	$filesel->destroy;
}
## cancel adding project to list
sub fsBatchAddProjectCancelButtonClicked
{
	my ($button,$widgets) = @_;
	$widgets->{'fsBatchAddProject'}->destroy;
}
## add project to list
sub fsBatchAddProjectOkButtonClicked
{
	my ($button,$widgets) = @_;
 	$widgets->{'fsBatchAddProject'}->hide;

	my @filenames = $widgets->{'fsBatchAddProject'}->file_list->selection;
	if( $#filenames > -1 )
	{
		for( my $i=0; $i<@filenames; ++$i )
		{
			my $txt = $widgets->{'fsBatchAddProject'}->selection_text->get;
			$txt = substr($txt,11);
			$txt .= "/".$widgets->{'fsBatchAddProject'}->file_list->get_text( $filenames[$i], 0 );
			my $item = new Gtk::ListItem( $txt );
			$item->show;
			$widgets->{'lstBatchList'}->add( $item );
		}
	}
	$widgets->{'fsBatchAddProject'}->destroy;
}

## remove selected project from list
sub btnBatchRemoveProjectClicked
{
	my ($button, $widgets) = @_;
	my @sel = $widgets->{'lstBatchList'}->selection;
	$widgets->{'lstBatchList'}->remove_items( @sel );
}
## remove all projects
sub btnBatchRemoveProjectsClicked
{
	my ($button, $widgets) = @_;
	my @children = $widgets->{'lstBatchList'}->children;
	$widgets->{'lstBatchList'}->remove_items( @children );
}

## start batch encoding
sub btnBatchStartClicked
{
	my ($button, $widgets) = @_;
	my @projects = $widgets->{'lstBatchList'}->children;

	if( $#projects > -1 )
	{
		if( GetBatchStatus() ne "started" )
		{
			$widgets->{'btnBatchStart'}->set_sensitive( $false );
			SetBatchStatus( "started" );
			for( my $i=0; $i<@projects; ++$i )
			{
				if( GetBatchStatus() eq "started" )
				{
					my $item = $projects[$i];
					my @itemchildren = $item->children;
					my $filenamelabel = $itemchildren[0];
					my $filename = $filenamelabel->get;
					
					$widgets->{'lblBatchActions'}->set_text( "Batch encoding $filename" );
					$widgets->{'lstBatchList'}->select_child( $item );

					ReadProject( $filename, $widgets );
					btnStartClicked($widgets->{'btnStart'}, $widgets);
				}
				else
				{
					$widgets->{'lblBatchActions'}->set_text( "Batch encoding stopped" );
				}
			}
			SetBatchStatus( "stopped" );
			$widgets->{'btnBatchStart'}->set_sensitive( $true );

		}
		else { $widgets->{'lblBatchActions'}->set_text( "Already busy !!" ) }
	}
}

## stop batch encoding
sub btnBatchStopClicked
{
	my ($button,$widgets) = @_;
	SetBatchStatus( "stopped" );
	btnStopClicked( $widgets->{'bntStop'}, $widgets );
}

## OutputFPS checkbutton toggled
sub chkbtnOutputFPSToggled
{
	my ($chkbtn, $widgets) = @_;
	if( $chkbtn->active )
	{
		$widgets->{'entryOutputFPS'}->set_sensitive( $true );
	}
	else
	{
		$widgets->{'entryOutputFPS'}->set_sensitive( $false );
	}
	entrySizeActivated( undef, $widgets );
	entryVideoBrateChanged( undef, $widgets );
}

## OutputFPS entry activated
sub entryOutputFPSActivated
{
	my ($entry,$widgets) = @_;
	if( $widgets->{'chkbtnOutputFPS'}->active )
	{
		entrySizeActivated( undef, $widgets );
		entryVideoBrateChanged( undef, $widgets );
	}
}

1;


